"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FsaCrud = void 0;
const util_1 = require("../node-to-fsa/util");
const util_2 = require("../crud/util");
const util_3 = require("./util");
class FsaCrud {
    constructor(root) {
        this.root = root;
        this.put = async (collection, id, data, options) => {
            (0, util_2.assertType)(collection, 'put', 'crudfs');
            (0, util_1.assertName)(id, 'put', 'crudfs');
            const [dir] = await this.getDir(collection, true);
            let file;
            switch (options?.throwIf) {
                case 'exists': {
                    try {
                        file = await dir.getFileHandle(id, { create: false });
                        throw (0, util_3.newExistsError)();
                    }
                    catch (e) {
                        if (e.name !== 'NotFoundError')
                            throw e;
                        file = await dir.getFileHandle(id, { create: true });
                    }
                    break;
                }
                case 'missing': {
                    try {
                        file = await dir.getFileHandle(id, { create: false });
                    }
                    catch (e) {
                        if (e.name === 'NotFoundError')
                            throw (0, util_3.newMissingError)();
                        throw e;
                    }
                    break;
                }
                default: {
                    file = await dir.getFileHandle(id, { create: true });
                }
            }
            const writable = await file.createWritable();
            await writable.write(data);
            await writable.close();
        };
        this.get = async (collection, id) => {
            (0, util_2.assertType)(collection, 'get', 'crudfs');
            (0, util_1.assertName)(id, 'get', 'crudfs');
            const [, file] = await this.getFile(collection, id);
            const blob = await file.getFile();
            const buffer = await blob.arrayBuffer();
            return new Uint8Array(buffer);
        };
        this.del = async (collection, id, silent) => {
            (0, util_2.assertType)(collection, 'del', 'crudfs');
            (0, util_1.assertName)(id, 'del', 'crudfs');
            try {
                const [dir] = await this.getFile(collection, id);
                await dir.removeEntry(id, { recursive: false });
            }
            catch (error) {
                if (!silent)
                    throw error;
            }
        };
        this.info = async (collection, id) => {
            (0, util_2.assertType)(collection, 'info', 'crudfs');
            if (id) {
                (0, util_1.assertName)(id, 'info', 'crudfs');
                const [, file] = await this.getFile(collection, id);
                const blob = await file.getFile();
                return {
                    type: 'resource',
                    id,
                    size: blob.size,
                    modified: blob.lastModified,
                };
            }
            else {
                await this.getDir(collection, false);
                return {
                    type: 'collection',
                    id: '',
                };
            }
        };
        this.drop = async (collection, silent) => {
            (0, util_2.assertType)(collection, 'drop', 'crudfs');
            try {
                const [dir, parent] = await this.getDir(collection, false);
                if (parent) {
                    await parent.removeEntry(dir.name, { recursive: true });
                }
                else {
                    const root = await this.root;
                    for await (const name of root.keys())
                        await root.removeEntry(name, { recursive: true });
                }
            }
            catch (error) {
                if (!silent)
                    throw error;
            }
        };
        this.scan = async function* (collection) {
            (0, util_2.assertType)(collection, 'scan', 'crudfs');
            const [dir] = await this.getDir(collection, false);
            for await (const [id, handle] of dir.entries()) {
                if (handle.kind === 'file') {
                    yield {
                        type: 'resource',
                        id,
                    };
                }
                else if (handle.kind === 'directory') {
                    yield {
                        type: 'collection',
                        id,
                    };
                }
            }
        };
        this.list = async (collection) => {
            const entries = [];
            for await (const entry of this.scan(collection))
                entries.push(entry);
            return entries;
        };
        this.from = async (collection) => {
            (0, util_2.assertType)(collection, 'from', 'crudfs');
            const [dir] = await this.getDir(collection, true);
            return new FsaCrud(dir);
        };
    }
    async getDir(collection, create) {
        let parent = undefined;
        let dir = await this.root;
        try {
            for (const name of collection) {
                const child = await dir.getDirectoryHandle(name, { create });
                parent = dir;
                dir = child;
            }
            return [dir, parent];
        }
        catch (error) {
            if (error.name === 'NotFoundError')
                throw (0, util_3.newFolder404Error)(collection);
            throw error;
        }
    }
    async getFile(collection, id) {
        const [dir] = await this.getDir(collection, false);
        try {
            const file = await dir.getFileHandle(id, { create: false });
            return [dir, file];
        }
        catch (error) {
            if (error.name === 'NotFoundError')
                throw (0, util_3.newFile404Error)(collection, id);
            throw error;
        }
    }
}
exports.FsaCrud = FsaCrud;
